==> dassadmin_2.59.tar.gz: dassadmin/src/de/dassit/ldap/Entry.java <==
/*
 * Copyright 2010 Sebastian Lederer <slederer@insignificance.de>
 */

package de.dassit.ldap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.regex.Pattern;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchResult;

public abstract class Entry {
	protected Attributes attributes;

	protected List<String> deletedAttributes;

	protected List<String> modifiedAttributes;

	protected String namingAttribute = "cn";

	protected String defaultNamingAttribute = "cn";

	protected static HashMap<String, String> containers = new HashMap<String, String>();

	protected static HashMap<String, String> namingAttributes = new HashMap<String, String>();

	protected static Pattern invalidCharacters = null;

	protected String container = "";

	protected String defaultContainer = null;

	String fullName; // dn relative to container

	String dn;

	String[] objectClasses;

	boolean isNew;

	public Entry() {
		attributes = new BasicAttributes(true);
		deletedAttributes = new ArrayList<String>();
		modifiedAttributes = new ArrayList<String>();
		defaultContainer = getDefaultContainer(this.getClass().getName());
		defaultNamingAttribute = getDefaultNamingAttribute(this.getClass()
				.getName());
		namingAttribute = defaultNamingAttribute;
		setNamingParameters();
		isNew = true;
	}

	public Entry(SearchResult sr) {
		deletedAttributes = new ArrayList<String>();
		modifiedAttributes = new ArrayList<String>();
		attributes = sr.getAttributes();
		fullName = sr.getName();
		dn = sr.getNameInNamespace();
		defaultNamingAttribute = getDefaultNamingAttribute(this.getClass()
				.getName());
		int i = fullName.indexOf('=');
		if (i > 0)
			namingAttribute = fullName.substring(0, i);
		setNamingParameters();
		isNew = false;
	}

	protected void setNamingParameters() {
	}

	/**
	 * @return the defaultNamingAttribute
	 */
	public String getDefaultNamingAttribute() {
		if (defaultNamingAttribute == null) {
			final String className = this.getClass().getName();
			defaultNamingAttribute = getDefaultNamingAttribute(className);
		}
		return defaultNamingAttribute;
	}

	/**
	 * @param defaultNamingAttribute
	 *            the defaultNamingAttribute to set
	 */
	public void setDefaultNamingAttribute(String defaultNamingAttribute) {
		this.defaultNamingAttribute = defaultNamingAttribute;
		namingAttributes.put(this.getClass().getName(), defaultNamingAttribute);
	}

	/**
	 * @return the defaultContainer
	 */
	public String getDefaultContainer() {
		if (defaultContainer == null) {
			final String className = this.getClass().getName();
			if (containers.containsKey(className)) {
				defaultContainer = containers.get(className);
			}
		}
		return defaultContainer;
	}

	/**
	 * @param defaultContainer
	 *            the defaultContainer to set
	 */
	public void setDefaultContainer(String defaultContainer) {
		this.defaultContainer = defaultContainer;
		containers.put(this.getClass().getName(), defaultContainer);
	}

	public static String getDefaultContainer(String className) {
		return containers.get(className);
	}

	public static String getDefaultNamingAttribute(String className) {
		if (namingAttributes.containsKey(className))
			return namingAttributes.get(className);
		else
			return "cn";
	}

	public void setInvalidCharacters(String s) {
		invalidCharacters = Pattern.compile(s);
	}

	public String getInvalidCharacters() {
		return invalidCharacters.pattern();
	}

	public void setContainer(String s) {
		container = s;
	}

	public String getContainer() {
		return container;
	}

	public void setObjectClasses(String[] objectClasses) {
		Attribute a = new BasicAttribute("objectClass", true);
		for (String s : objectClasses) {
			a.add(s);
		}
		attributes.put(a);
	}

	public void addObjectClass(String o) {
		appendAttributeValue("objectClass", o);
	}

	public void addObjectClassesCSV(String csv) {
		if (csv == null || csv.equals(""))
			return;

		for (String s : csv.split(",")) {
			addObjectClass(s);
		}
	}

	public void setNamingAttribute(String n) {
		namingAttribute = n;
	}

	Attributes getAttributes() {
		return attributes;
	}

	public String getNamingAttribute() {
		return namingAttribute;
	}

	public String get(String key) {
		return getAttribute(key);
	}

	/**
	 * stores an attribute. If value is null or an emtpy string, the attribute
	 * is removed.
	 */
	public void put(String key, String value) {
		if (value.equals("")) {
			value = null;
		}

		String oldValue = getAttribute(key);

		if (oldValue == null && value == null)
			return;
		if (oldValue != null && value != null && value.equals(oldValue))
			return;

		setAttribute(key, value);
	}

	public void validateAttributeValue(String a, String s) {
		if (a != null && s != null && invalidCharacters != null
				&& invalidCharacters.matcher(s).matches()) {
			throw new IllegalArgumentException(
					"invalid characters in attribute " + a);
		}
	}

	public void setAttribute(String key, String value) {
		validateAttributeValue(key, value);
		Attribute a;
		a = attributes.get(key);

		if (value == null) {
			if (a != null)
				deleteAttribute(key);
			return;
		}

		if (a == null) {
			a = new BasicAttribute(key, false);
			a.add(value);
			attributes.put(a);
			// System.out.println("setting " + key + "=" + value);
		} else {
			a.clear();
			a.add(value);
			// System.out.println("replacing " + key + "=" + value);
		}
		if (!modifiedAttributes.contains(key))
			modifiedAttributes.add(key);
		if (deletedAttributes.contains(key))
			deletedAttributes.remove(key);
	}

	public void deleteAttribute(String key) {
		if (attributes.get(key) == null)
			return;

		attributes.remove(key);
		modifiedAttributes.remove(key);
		deletedAttributes.add(key);
	}

	public void appendAttributeValue(String key, String value) {
		Attribute a;
		a = attributes.get(key);
		if (a == null) {
			setAttribute(key, value);
		} else {
			if (!modifiedAttributes.contains(key))
				modifiedAttributes.add(key);
			a.add(value);
		}
	}

	public void removeAttributeValue(String key, String value) {
		Attribute a = attributes.get(key);

		if (a == null)
			return;
		if (!modifiedAttributes.contains(key))
			modifiedAttributes.add(key);
		a.remove(value);
	}

	public void setName(String name) {
		setAttribute(namingAttribute, name);
	}

	public String getFullName() {
		return fullName;
	}

	public void setFullName(String fullName) {
		this.fullName = fullName;
	}

	public String getRDN() {
		if (container == null || container.equals(""))
			return fullName;
		return fullName + "," + container;
	}

	public String getName() {
		return getAttribute(namingAttribute);
	}

	public String getDN() {
		return dn;
	}

	void setDN(String d) {
		dn = d;
	}

	public void saveAsNew() {
		isNew = true;
	}

	public String getAttribute(String attrName) throws NoSuchElementException {
		Attribute a = attributes.get(attrName);
		if (attrName == null || a == null)
			return null;

		try {
			return a.get().toString();
		} catch (NamingException e) {
			e.printStackTrace();
			throw new NoSuchElementException(
					"failed to retrieve attribute value for " + attrName);
		}
	}

	public String getAttribute(String attrName, int index)
			throws NoSuchElementException {
		Attribute a = attributes.get(attrName);
		if (attrName == null)
			throw new NoSuchElementException("attribute " + attrName
					+ " not found");

		if (a.size() <= index)
			throw new NoSuchElementException("illegal attribute value index "
					+ index + " for " + attrName + ", valid range 0-"
					+ (a.size() - 1));

		try {
			return (String) a.get(index);
		} catch (NamingException e) {
			e.printStackTrace();
			throw new NoSuchElementException(
					"failed to retrieve attribute value for " + attrName);
		}
	}

	public int getAttributeSize(String attrName) throws NoSuchElementException {
		if (attrName == null)
			throw new NoSuchElementException("attribute " + attrName
					+ " not found");
		Attribute a = attributes.get(attrName);
		if (a == null)
			return 0;
		else
			return a.size();
	}

	public ArrayList<String> getAttributeMultiValue(String attrName)
			throws NoSuchElementException {
		if (attrName == null)
			throw new NoSuchElementException("attribute " + attrName
					+ " not found");
		Attribute a = attributes.get(attrName);
		if (a == null)
			return null;
		ArrayList<String> result = new ArrayList<String>();
		NamingEnumeration<? extends Object> en;
		try {
			en = a.getAll();
			while (en.hasMore())
				result.add((String) en.next());
		} catch (NamingException e) {
			return null;
		}
		return result;
	}

	public void delete() throws NamingException {
		delete(Connection.getDefaultConnection());
	}

	public void delete(Connection c) throws NamingException {
		if (!isNew) {
			c.deleteEntry(this);
		}
	}

	/**
	 * Renames the object, immediately affecting the backing object. After the
	 * rename operation, the in-memory object becomes invalid and should not be
	 * used anymore. Calls {@link rename(String,Connection)} with the default
	 * connection.
	 * 
	 * @param newName
	 *            the new name for the object - becomes the attribute value of
	 *            the naming attribute
	 * @throws NamingException
	 * @see #rename(String,Connection)
	 */
	public void rename(String newName) throws NamingException {
		rename(Connection.getDefaultConnection(), newName);
	}

	/**
	 * Renames the object, immediately affecting the backing object. After the
	 * rename operation, the in-memory object becomes invalid and should not be
	 * used anymore. Objects can be moved to a different container by setting
	 * the container name before calling rename().
	 * 
	 * @param c
	 * @param newName
	 *            the new name for the object - becomes the attribute value of
	 *            the naming attribute
	 * @throws NamingException
	 */
	public void rename(Connection c, String newName) throws NamingException {
		String suffix = "";
		int i = fullName.indexOf(',');
		if (i > 0) {
			suffix = "," + fullName.substring(i + 1);
		}
		String newFullName = namingAttribute + "=" + newName + suffix + ","
				+ container;
		// System.out.println("newFullName:" + newFullName);
		c.renameEntry(this, newFullName);

		fullName = newFullName;
	}

	public void save() throws NamingException {
		save(Connection.getDefaultConnection());
	}

	public void save(Connection c) throws NamingException {
		if (isNew) {
			c.addEntry(this, container);
		} else {
			int i = 0;
			int modCount = modifiedAttributes.size() + deletedAttributes.size();

			if (modCount == 0)
				return;

			ModificationItem mods[] = new ModificationItem[modCount];

			for (String key : modifiedAttributes) {
				mods[i++] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
						attributes.get(key));
			}

			for (String key : deletedAttributes) {
				mods[i++] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE,
						new BasicAttribute(key));
			}

			c.modifyEntry(this, mods);
		}

	}

	public void printAttrs() {
		try {
			for (NamingEnumeration<? extends Attribute> ae = attributes
					.getAll(); ae.hasMore();) {
				Attribute attr = (Attribute) ae.next();
				System.out.println("attribute: " + attr.getID());
				/* Print each value */
				for (NamingEnumeration<?> e = attributes.getAll(); e.hasMore(); System.out
						.println("value: " + e.next()))
					;
			}
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * <code>return new EntryList<Entry>() {
	 * 		public void addInstance(SearchResult sr) {
	 * 			entries.add(new Entry(sr));
	 * 		}
	 * 	};
	 * 	</code>
	 * 
	 * @return
	 */
	abstract protected EntryList<? extends Entry> getEntryList();

	public Entry fetchByDN(String dn) {
		EntryList<? extends Entry> results = getEntryList();
		Connection.getDefaultConnection().getByDN(dn, results);
		if (results.size() == 0)
			return null;
		else
			return results.entries.get(0);
	}

	public Entry fetch(String key) {
		final String filter = defaultNamingAttribute + "=" + key;
		List<? extends Entry> result = search(filter);
		if (result.size() > 0)
			return result.get(0);
		else
			return null;
	}

	public List<? extends Entry> search(String filter, String container) {

		EntryList<? extends Entry> entryList = getEntryList();

		Connection.getDefaultConnection().search(filter, container, entryList);

		return entryList.entries;
	}

	public List<? extends Entry> search(String filter) {
		return search(filter, defaultContainer);
	}

	public List<? extends Entry> getAll() {
		return search("objectClass=*", defaultContainer);
	}

	@Override
	public String toString() {
		return attributes.toString();
	}

}

==> dassadmin_2.62.tar.gz: dassadmin/src/de/dassit/ldap/Entry.java <==
/*
 * Copyright 2010 Sebastian Lederer <slederer@insignificance.de>
 */

package de.dassit.ldap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.regex.Pattern;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchResult;

public abstract class Entry {
	protected Attributes attributes;

	protected List<String> deletedAttributes;

	protected List<String> modifiedAttributes;

	protected String namingAttribute = "cn";

	protected String defaultNamingAttribute = "cn";

	protected static HashMap<String, String> containers = new HashMap<String, String>();

	protected static HashMap<String, String> namingAttributes = new HashMap<String, String>();

	protected static Pattern invalidCharacters = null;

	protected String container = "";

	protected String defaultContainer = null;

	String fullName; // dn relative to container

	String dn;

	String[] objectClasses;

	boolean isNew;

	Connection connection = null;

	public Entry() {
		attributes = new BasicAttributes(true);
		deletedAttributes = new ArrayList<String>();
		modifiedAttributes = new ArrayList<String>();
		defaultContainer = getDefaultContainer(this.getClass().getName());
		defaultNamingAttribute = getDefaultNamingAttribute(this.getClass()
				.getName());
		namingAttribute = defaultNamingAttribute;
		setNamingParameters();
		isNew = true;
	}

	public Entry(SearchResult sr) {
		deletedAttributes = new ArrayList<String>();
		modifiedAttributes = new ArrayList<String>();
		attributes = sr.getAttributes();
		fullName = sr.getName();
		dn = sr.getNameInNamespace();
		defaultNamingAttribute = getDefaultNamingAttribute(this.getClass()
				.getName());
		int i = fullName.indexOf('=');
		if (i > 0)
			namingAttribute = fullName.substring(0, i);
		setNamingParameters();
		isNew = false;
	}

	protected void setNamingParameters() {
	}

	/**
	 * @return the defaultNamingAttribute
	 */
	public String getDefaultNamingAttribute() {
		if (defaultNamingAttribute == null) {
			final String className = this.getClass().getName();
			defaultNamingAttribute = getDefaultNamingAttribute(className);
		}
		return defaultNamingAttribute;
	}

	/**
	 * @param defaultNamingAttribute
	 *            the defaultNamingAttribute to set
	 */
	public void setDefaultNamingAttribute(String defaultNamingAttribute) {
		this.defaultNamingAttribute = defaultNamingAttribute;
		namingAttributes.put(this.getClass().getName(), defaultNamingAttribute);
	}

	/**
	 * @return the defaultContainer
	 */
	public String getDefaultContainer() {
		if (defaultContainer == null) {
			final String className = this.getClass().getName();
			if (containers.containsKey(className)) {
				defaultContainer = containers.get(className);
			}
		}
		return defaultContainer;
	}

	/**
	 * @param defaultContainer
	 *            the defaultContainer to set
	 */
	public void setDefaultContainer(String defaultContainer) {
		this.defaultContainer = defaultContainer;
		containers.put(this.getClass().getName(), defaultContainer);
	}

	public static String getDefaultContainer(String className) {
		return containers.get(className);
	}

	public static String getDefaultNamingAttribute(String className) {
		if (namingAttributes.containsKey(className))
			return namingAttributes.get(className);
		else
			return "cn";
	}

	public void setInvalidCharacters(String s) {
		invalidCharacters = Pattern.compile(s);
	}

	public String getInvalidCharacters() {
		return invalidCharacters.pattern();
	}

	public void setContainer(String s) {
		container = s;
	}

	public String getContainer() {
		return container;
	}

	public void setObjectClasses(String[] objectClasses) {
		Attribute a = new BasicAttribute("objectClass", true);
		for (String s : objectClasses) {
			a.add(s);
		}
		attributes.put(a);
	}

	public void addObjectClass(String o) {
		appendAttributeValue("objectClass", o);
	}

	public void addObjectClassesCSV(String csv) {
		if (csv == null || csv.equals(""))
			return;

		for (String s : csv.split(",")) {
			addObjectClass(s);
		}
	}

	public void setNamingAttribute(String n) {
		namingAttribute = n;
	}

	public Attributes getAttributes() {
		return attributes;
	}

	public List<String> getAttributeNames() {
		ArrayList<String> result = new ArrayList<String>(attributes.size());

		NamingEnumeration<? extends Attribute> e = attributes.getAll();
		try {
			while (e.hasMore()) {
				Attribute a = (Attribute) e.next();
				String name = translate(a.getID());
				result.add(name);
			}
		} catch (NamingException e1) {
			e1.printStackTrace();
		}

		return result;
	}

	public String getNamingAttribute() {
		return namingAttribute;
	}

	public String get(String key) {
		return getAttribute(key);
	}

	/**
	 * stores an attribute. If value is null or an emtpy string, the attribute
	 * is removed.
	 */
	public void put(String key, String value) {
		if (value.equals("")) {
			value = null;
		}

		String oldValue = getAttribute(key);

		if (oldValue == null && value == null)
			return;
		if (oldValue != null && value != null && value.equals(oldValue))
			return;

		setAttribute(key, value);
	}

	public void validateAttributeValue(String a, String s) {
		if (a != null && s != null && invalidCharacters != null
				&& invalidCharacters.matcher(s).matches()) {
			throw new IllegalArgumentException(
					"invalid characters in attribute " + a);
		}
	}

	public void setAttribute(String key, String value) {
		validateAttributeValue(key, value);
		Attribute a = attributes.get(key);

		if (value == null) {
			if (a != null)
				deleteAttribute(key);
			return;
		}

		if (a == null) {
			a = new BasicAttribute(key, false);
			a.add(value);
			attributes.put(a);
			// System.out.println("setting " + key + "=" + value);
		} else {
			a.clear();
			a.add(value);
			// System.out.println("replacing " + key + "=" + value);
		}
		if (!modifiedAttributes.contains(key))
			modifiedAttributes.add(key);
		if (deletedAttributes.contains(key))
			deletedAttributes.remove(key);
	}

	public void deleteAttribute(String key) {
		if (attributes.get(key) == null)
			return;

		attributes.remove(key);
		modifiedAttributes.remove(key);
		deletedAttributes.add(key);
	}

	public void appendAttributeValue(String key, String value) {
		Attribute a;
		a = attributes.get(key);
		if (a == null) {
			setAttribute(key, value);
		} else {
			if (!modifiedAttributes.contains(key))
				modifiedAttributes.add(key);
			a.add(value);
		}
	}

	public void removeAttributeValue(String key, String value) {
		Attribute a = attributes.get(key);

		if (a == null)
			return;
		if (!modifiedAttributes.contains(key))
			modifiedAttributes.add(key);
		a.remove(value);
	}

	public void setName(String name) {
		setAttribute(namingAttribute, name);
	}

	public String getFullName() {
		return fullName;
	}

	public void setFullName(String fullName) {
		this.fullName = fullName;
	}

	public String getRDN() {
		if (container == null || container.equals(""))
			return fullName;
		return fullName + "," + container;
	}

	public String getName() {
		return getAttribute(namingAttribute);
	}

	public String getDN() {
		return dn;
	}

	void setDN(String d) {
		dn = d;
	}

	public void saveAsNew() {
		isNew = true;
	}

	public String getAttribute(String attrName) throws NoSuchElementException {
		Attribute a = attributes.get(attrName);
		if (attrName == null || a == null)
			return null;

		try {
			return a.get().toString();
		} catch (NamingException e) {
			e.printStackTrace();
			throw new NoSuchElementException(
					"failed to retrieve attribute value for " + attrName);
		}
	}

	public String getAttribute(String attrName, int index)
			throws NoSuchElementException {
		Attribute a = attributes.get(attrName);
		if (attrName == null)
			throw new NoSuchElementException("attribute " + attrName
					+ " not found");

		if (a.size() <= index)
			throw new NoSuchElementException("illegal attribute value index "
					+ index + " for " + attrName + ", valid range 0-"
					+ (a.size() - 1));

		try {
			return (String) a.get(index);
		} catch (NamingException e) {
			e.printStackTrace();
			throw new NoSuchElementException(
					"failed to retrieve attribute value for " + attrName);
		}
	}

	public int getAttributeSize(String attrName) throws NoSuchElementException {
		if (attrName == null)
			throw new NoSuchElementException("attribute " + attrName
					+ " not found");
		Attribute a = attributes.get(attrName);
		if (a == null)
			return 0;
		else
			return a.size();
	}

	public ArrayList<String> getAttributeMultiValue(String attrName)
			throws NoSuchElementException {
		if (attrName == null)
			throw new NoSuchElementException("attribute " + attrName
					+ " not found");
		Attribute a = attributes.get(attrName);
		if (a == null)
			return null;
		ArrayList<String> result = new ArrayList<String>();
		NamingEnumeration<? extends Object> en;
		try {
			en = a.getAll();
			while (en.hasMore())
				result.add((String) en.next());
		} catch (NamingException e) {
			return null;
		}
		return result;
	}

	public void delete() throws NamingException {
		delete(getConnection());
	}

	public void delete(Connection c) throws NamingException {
		if (!isNew) {
			c.deleteEntry(this);
		}
	}

	/**
	 * Renames the object, immediately affecting the backing object. After the
	 * rename operation, the in-memory object becomes invalid and should not be
	 * used anymore. Calls {@link rename(String,Connection)} with the default
	 * connection.
	 * 
	 * @param newName
	 *            the new name for the object - becomes the attribute value of
	 *            the naming attribute
	 * @throws NamingException
	 * @see #rename(String,Connection)
	 */
	public void rename(String newName) throws NamingException {
		rename(getConnection(), newName);
	}

	/**
	 * Renames the object, immediately affecting the backing object. After the
	 * rename operation, the in-memory object becomes invalid and should not be
	 * used anymore. Objects can be moved to a different container by setting
	 * the container name before calling rename().
	 * 
	 * @param c
	 * @param newName
	 *            the new name for the object - becomes the attribute value of
	 *            the naming attribute
	 * @throws NamingException
	 */
	public void rename(Connection c, String newName) throws NamingException {
		String suffix = "";
		int i = fullName.indexOf(',');
		if (i > 0) {
			suffix = "," + fullName.substring(i + 1);
		}
		String newFullName = namingAttribute + "=" + newName + suffix + ","
				+ container;
		// System.out.println("newFullName:" + newFullName);
		c.renameEntry(this, newFullName);

		fullName = newFullName;
	}

	public void save() throws NamingException {
		save(getConnection());
	}

	public void save(Connection c) throws NamingException {
		if (isNew) {
			c.addEntry(this, container);
		} else {
			int i = 0;
			int modCount = modifiedAttributes.size() + deletedAttributes.size();

			if (modCount == 0)
				return;

			ModificationItem mods[] = new ModificationItem[modCount];

			for (String key : modifiedAttributes) {
				mods[i++] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
						attributes.get(key));
			}

			for (String key : deletedAttributes) {
				mods[i++] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE,
						new BasicAttribute(key));
			}

			c.modifyEntry(this, mods);
		}

	}

	public void printAttrs() {
		try {
			for (NamingEnumeration<? extends Attribute> ae = attributes
					.getAll(); ae.hasMore();) {
				Attribute attr = (Attribute) ae.next();
				System.out.println("attribute: " + attr.getID());
				/* Print each value */
				for (NamingEnumeration<?> e = attributes.getAll(); e.hasMore(); System.out
						.println("value: " + e.next()))
					;
			}
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * <code>return new EntryList<Entry>() {
	 * 		public void addInstance(SearchResult sr) {
	 * 			entries.add(new Entry(sr));
	 * 		}
	 * 	};
	 * 	</code>
	 * 
	 * @return
	 */
	abstract protected EntryList<? extends Entry> getEntryList();

	public Entry fetchByDN(String dn) {
		EntryList<? extends Entry> results = getEntryList();
		getConnection().getByDN(dn, results);
		if (results.size() == 0)
			return null;
		else
			return results.entries.get(0);
	}

	public Entry fetch(String key) {
		final String filter = defaultNamingAttribute + "=" + key;
		List<? extends Entry> result = search(filter);
		if (result.size() > 0)
			return result.get(0);
		else
			return null;
	}

	/**
	 * returns the connection. If no connection has been set,
	 * returns the default connection from the Connection class.
	 * 
	 * @return a connection
	 */
	public Connection getConnection() {
		if (connection != null)
			return connection;

		return Connection.getDefaultConnection();
	}

	/**
	 * @param connection
	 *            the connection to set
	 */
	public void setConnection(Connection connection) {
		this.connection = connection;
	}

	public List<? extends Entry> search(String filter, String container) {

		EntryList<? extends Entry> entryList = getEntryList();

		getConnection().search(filter, container, entryList);

		return entryList.entries;
	}

	public List<? extends Entry> search(String filter) {
		return search(filter, defaultContainer);
	}

	public List<? extends Entry> getAll() {
		return search("objectClass=*", defaultContainer);
	}

	@Override
	public String toString() {
		return attributes.toString();
	}

	public boolean isMultivalue(String attributeName) {
		if (attributeName.equals("objectClass"))
			return true;
		return false;
	}

	public boolean isInteger(String attributeName) {
		return false;
	}

	public boolean isBoolean(String attributeName) {
		return false;
	}

	public boolean isReadOnly(String attributeName) {
		return false;
	}

	public boolean isHidden(String attributeName) {
		if (attributeName.equals("objectClass"))
			return true;

		return false;
	}

	/**
	 * Translate LDAP attribute name to Entry attribute name <br>
	 * Example: A User object (posixAccount) has the LDAP attribute uid, which
	 * is mapped to the getName/setName methods. So "uid" is translated to
	 * "name".
	 * 
	 * @param attributeName
	 *            LDAP attribute name
	 * @return Entry attribute name
	 */
	public String translate(String attributeName) {
		if (attributeName.equals(namingAttribute))
			return "name";
		return attributeName;
	}
}
